]> git.sesse.net Git - nageru/blobdiff - mixer.cpp
Make the level compressor much slower.
[nageru] / mixer.cpp
index 03cf99d3435969891ccc1961d7f71c1b479861a5..506af4aeabb95fa8d554cc32baf80dfe5c61b3ff 100644 (file)
--- a/mixer.cpp
+++ b/mixer.cpp
@@ -384,7 +384,8 @@ void Mixer::thread_func()
                        double loudness_range_high = r128.range_max();
 
                        audio_level_callback(loudness_s, 20.0 * log10(peak),
-                                            loudness_i, loudness_range_low, loudness_range_high);
+                                            loudness_i, loudness_range_low, loudness_range_high,
+                                            last_gain_staging_db);
                }
 
                for (unsigned card_index = 1; card_index < num_cards; ++card_index) {
@@ -548,28 +549,25 @@ void Mixer::process_audio_one_frame()
        // Apply a level compressor to get the general level right.
        // Basically, if it's over about -40 dBFS, we squeeze it down to that level
        // (or more precisely, near it, since we don't use infinite ratio),
-       // then apply a makeup gain to get it to -12 dBFS. -12 dBFS is, of course,
+       // then apply a makeup gain to get it to -14 dBFS. -14 dBFS is, of course,
        // entirely arbitrary, but from practical tests with speech, it seems to
        // put ut around -23 LUFS, so it's a reasonable starting point for later use.
        //
-       // TODO: Hook this up to a UI, so we can see the effects, and/or turn it off
-       // to control the gain manually instead. For now, there's only the #if-ed out
-       // code below.
-       //
        // TODO: Add the actual compressors/limiters (for taking care of transients)
        // later in the chain.
        float threshold = 0.01f;   // -40 dBFS.
        float ratio = 20.0f;
-       float attack_time = 0.1f;
-       float release_time = 10.0f;
-       float makeup_gain = pow(10.0f, 28.0f / 20.0f);  // +28 dB takes us to -12 dBFS.
+       float attack_time = 0.5f;
+       float release_time = 20.0f;
+       float makeup_gain = pow(10.0f, 26.0f / 20.0f);  // +26 dB takes us to -14 dBFS.
        level_compressor.process(samples_out.data(), samples_out.size() / 2, threshold, ratio, attack_time, release_time, makeup_gain);
+       last_gain_staging_db = 20.0 * log10(level_compressor.get_attenuation() * makeup_gain);
 
 #if 0
        printf("level=%f (%+5.2f dBFS) attenuation=%f (%+5.2f dB) end_result=%+5.2f dB\n",
-               compressor.get_level(), 20.0 * log10(compressor.get_level()),
-               compressor.get_attenuation(), 20.0 * log10(compressor.get_attenuation()),
-               20.0 * log10(compressor.get_level() * compressor.get_attenuation() * makeup_gain));
+               level_compressor.get_level(), 20.0 * log10(level_compressor.get_level()),
+               level_compressor.get_attenuation(), 20.0 * log10(level_compressor.get_attenuation()),
+               20.0 * log10(level_compressor.get_level() * level_compressor.get_attenuation() * makeup_gain));
 #endif
 
        // Find peak and R128 levels.